/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.16.1.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP_UART.c
Purpose : UART implementation

Additional information:

  Device : TI MSP430F149
  Board  : TI MSP-TS430PM64

  Unit | UART    | Board connector
  ================================
  0    | UART0   | P3.4, P3.5

*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_INT_Enter()/OS_INT_Leave(). Remove this line and OS_INT_* functions if not using OS.

#include <msp430.h>   // Device specific header file, contains CMSIS defines.


/*********************************************************************
*
*       Defines
*
**********************************************************************
*/


/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  //
  // Initialize baudrate generator and modulation control
  //
  if (Unit == 0) {
    if (Baudrate == 9600) {
      U0BR0  = 0x03;
      U0BR1  = 0x00;
      U0MCTL = 0x4A;
    } else if (Baudrate == 4800) {
      U0BR0  = 0x06;
      U0BR1  = 0x00;
      U0MCTL = 0x6F;
    } else {
      for (;;);  // Selected baudrate not supported
    }
  }
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       UART_TX_IRQHandler()
*
*  Function description
*    UART Tx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are in interrupt context.
*/
#pragma vector=USART0TX_VECTOR
static __interrupt void UART_TX_IRQHandler(void) {
  OS_INT_Enter();
  if (_pfWriteCB != NULL) {
    _pfWriteCB(0);  // Tell the callback that we are ready to send the next byte.
  }
  OS_INT_Leave();
}

/*********************************************************************
*
*       UART_RX_IRQHandler()
*
*  Function description
*    UART Rx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are in interrupt context.
*/
#pragma vector=USART0RX_VECTOR
static __interrupt void UART_RX_IRQHandler(void) {
  OS_INT_Enter();
  if (_pfReadCB != NULL) {
    _pfReadCB(0, U0RXBUF);  // Forward the received byte to the callback.
  }
  OS_INT_Leave();
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  if (Unit == 0) {
    U0CTL  = SWRST;              // Reset UART, No parity, 1 stop bit
    P3SEL |= (BIT4 | BIT5);      // Set Port function to UART mode
    U0CTL |= CHAR;               // Set 8 bit data length
    U0TCTL = 0x10;               // Set tx mode
    U0RCTL = 0;                  // Reset Rx Control, no wake int.
    _SetBaudrate(Unit, Baudrate);
    ME1   |= (URXE0 | UTXE0);    // Enable Rx Tx
    U0CTL &= ~SWRST;             // Unlock UART
    IE1   |= (UTXIE0 | URXIE0);  // Enable Rx Tx interrupts
  }
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Data);
  U0TXBUF = Data;
}

/*************************** End of file ****************************/
